# Copyright (c) 2014 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import exceptions

from mistralclient.tests.functional.cli.v2 import base_v2


class StandardItemsAvailabilityCLITests(base_v2.MistralClientTestBase):

    def test_std_workflows_availability(self):
        wfs = self.mistral_admin("workflow-list")

        self.assertTableStruct(
            wfs,
            ["Name", "Tags", "Input", "Created at", "Updated at"]
        )
        self.assertPartIn("create_instance",
                          [workflow["Name"] for workflow in wfs])

        wfs = self.mistral_alt_user("workflow-list")

        self.assertTableStruct(
            wfs,
            ["Name", "Tags", "Input", "Created at", "Updated at"]
        )
        self.assertPartIn("create_instance",
                          [workflow["Name"] for workflow in wfs])

    def test_std_actions_availability(self):
        acts = self.mistral_admin("action-list")

        self.assertTableStruct(
            acts,
            ["Name", "Is system", "Input", "Description",
             "Tags", "Created at", "Updated at"]
        )
        self.assertIn("glance.images_list",
                      [action["Name"] for action in acts])

        acts = self.mistral_alt_user("action-list")

        self.assertTableStruct(
            acts,
            ["Name", "Is system", "Input", "Description",
             "Tags", "Created at", "Updated at"]
        )
        self.assertIn("glance.images_list",
                      [action["Name"] for action in acts])


class WorkbookIsolationCLITests(base_v2.MistralClientTestBase):

    def test_workbook_name_uniqueness(self):
        self.workbook_create(self.wb_def)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_admin,
            "workbook-create",
            params="{0}".format(self.wb_def)
        )

        self.workbook_create(self.wb_def, admin=False)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "workbook-create",
            params="{0}".format(self.wb_def)
        )

    def test_wb_isolation(self):
        wb = self.workbook_create(self.wb_def)
        wb_name = self.get_field_value(wb, "Name")
        wbs = self.mistral_admin("workbook-list")

        self.assertIn(wb_name, [w["Name"] for w in wbs])

        alt_wbs = self.mistral_alt_user("workbook-list")

        self.assertNotIn(wb_name, [w["Name"] for w in alt_wbs])

    def test_get_wb_from_another_tenant(self):
        wb = self.workbook_create(self.wb_def)
        name = self.get_field_value(wb, "Name")

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "workbook-get",
            params=name
        )

    def test_create_public_workbook(self):
        wb = self.workbook_create(self.wb_def, scope='public')
        name = self.get_field_value(wb, "Name")

        same_wb = self.mistral_alt_user(
            "workbook-get",
            params=name
        )

        self.assertEqual(
            name,
            self.get_field_value(same_wb, "Name")
        )

        # The workflows should be public too
        self.mistral_alt_user(
            "workflow-get",
            params="wb.wf1"
        )

        # The actions should be public too
        self.mistral_alt_user(
            "action-get",
            params="wb.ac1"
        )

    def test_delete_wb_from_another_tenant(self):
        wb = self.workbook_create(self.wb_def)
        name = self.get_field_value(wb, "Name")

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "workbook-delete",
            params=name
        )


class WorkflowIsolationCLITests(base_v2.MistralClientTestBase):

    def test_workflow_name_uniqueness(self):
        self.workflow_create(self.wf_def)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_admin,
            "workflow-create",
            params="{0}".format(self.wf_def)
        )

        self.workflow_create(self.wf_def, admin=False)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "workflow-create",
            params="{0}".format(self.wf_def)
        )

    def test_wf_isolation(self):
        wf = self.workflow_create(self.wf_def)
        wfs = self.mistral_admin("workflow-list")

        self.assertIn(wf[0]["Name"], [w["Name"] for w in wfs])

        alt_wfs = self.mistral_alt_user("workflow-list")

        self.assertNotIn(wf[0]["Name"], [w["Name"] for w in alt_wfs])

    def test_get_wf_from_another_tenant(self):
        wf = self.workflow_create(self.wf_def)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "workflow-get",
            params=wf[0]["ID"]
        )

    def test_create_public_workflow(self):
        wf = self.workflow_create(self.wf_def, scope='public')

        same_wf = self.mistral_alt_user(
            "workflow-get",
            params=wf[0]["Name"]
        )

        self.assertEqual(
            wf[0]["Name"],
            self.get_field_value(same_wf, "Name")
        )

    def test_delete_wf_from_another_tenant(self):
        wf = self.workflow_create(self.wf_def)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "workflow-delete",
            params=wf[0]["ID"]
        )


class WorkflowSharingCLITests(base_v2.MistralClientTestBase):
    def setUp(self):
        super(WorkflowSharingCLITests, self).setUp()

        self.wf = self.workflow_create(self.wf_def, admin=True)

    def _update_shared_workflow(self, new_status='accepted'):
        member = self.workflow_member_create(self.wf[0]["ID"])
        status = self.get_field_value(member, 'Status')

        self.assertEqual('pending', status)

        cmd_param = '%s workflow --status %s --member-id %s' % (
            self.wf[0]["ID"], new_status, self.get_project_id("alt_demo"))
        member = self.mistral_alt_user("member-update", params=cmd_param)
        status = self.get_field_value(member, 'Status')

        self.assertEqual(new_status, status)

    def test_list_accepted_shared_workflow(self):
        wfs = self.mistral_alt_user("workflow-list")

        self.assertNotIn(self.wf[0]["ID"], [w["ID"] for w in wfs])

        self._update_shared_workflow(new_status='accepted')
        alt_wfs = self.mistral_alt_user("workflow-list")

        self.assertIn(self.wf[0]["ID"], [w["ID"] for w in alt_wfs])
        self.assertIn(
            self.get_project_id("admin"),
            [w["Project ID"] for w in alt_wfs]
        )

    def test_list_rejected_shared_workflow(self):
        self._update_shared_workflow(new_status='rejected')
        alt_wfs = self.mistral_alt_user("workflow-list")

        self.assertNotIn(self.wf[0]["ID"], [w["ID"] for w in alt_wfs])

    def test_create_execution_using_shared_workflow(self):
        self._update_shared_workflow(new_status='accepted')

        execution = self.execution_create(self.wf[0]["ID"], admin=False)
        wf_name = self.get_field_value(execution, 'Workflow name')

        self.assertEqual(self.wf[0]["Name"], wf_name)

    def test_create_contrigger_using_shared_workflow(self):
        self._update_shared_workflow(new_status='accepted')

        trigger = self.cron_trigger_create(
            "test_trigger",
            self.wf[0]["ID"],
            "{}",
            "5 * * * *",
            admin=False
        )
        wf_name = self.get_field_value(trigger, 'Workflow')

        self.assertEqual(self.wf[0]["Name"], wf_name)

        # Admin project can not delete the shared workflow, because it is used
        # in a cron-trigger of another project.
        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_admin,
            'workflow-delete',
            params=self.wf[0]["ID"]
        )


class ActionIsolationCLITests(base_v2.MistralClientTestBase):

    def test_actions_name_uniqueness(self):
        self.action_create(self.act_def)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_admin,
            "action-create",
            params="{0}".format(self.act_def)
        )

        self.action_create(self.act_def, admin=False)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "action-create",
            params="{0}".format(self.act_def)
        )

    def test_action_isolation(self):
        act = self.action_create(self.act_def)
        acts = self.mistral_admin("action-list")

        self.assertIn(act[0]["Name"], [a["Name"] for a in acts])

        alt_acts = self.mistral_alt_user("action-list")

        self.assertNotIn(act[0]["Name"], [a["Name"] for a in alt_acts])

    def test_get_action_from_another_tenant(self):
        act = self.action_create(self.act_def)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "action-get",
            params=act[0]["Name"]
        )

    def test_delete_action_from_another_tenant(self):
        act = self.action_create(self.act_def)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "action-delete",
            params=act[0]["Name"]
        )

    def test_create_public_action(self):
        act = self.action_create(self.act_def, scope='public')

        same_act = self.mistral_alt_user(
            "action-get",
            params=act[0]["Name"]
        )

        self.assertEqual(
            act[0]["Name"],
            self.get_field_value(same_act, "Name")
        )


class CronTriggerIsolationCLITests(base_v2.MistralClientTestBase):
    def test_cron_trigger_name_uniqueness(self):
        wf = self.workflow_create(self.wf_def)
        self.cron_trigger_create(
            "admin_trigger",
            wf[0]["ID"],
            "{}",
            "5 * * * *"
        )

        self.assertRaises(
            exceptions.CommandFailed,
            self.cron_trigger_create,
            "admin_trigger",
            wf[0]["ID"],
            "{}"
            "5 * * * *",
        )

        wf = self.workflow_create(self.wf_def, admin=False)
        self.cron_trigger_create(
            "user_trigger",
            wf[0]["ID"],
            "{}",
            "5 * * * *",
            None,
            None,
            admin=False
        )

        self.assertRaises(
            exceptions.CommandFailed,
            self.cron_trigger_create,
            "user_trigger",
            wf[0]["ID"],
            "{}",
            "5 * * * *",
            None,
            None,
            admin=False
        )

    def test_cron_trigger_isolation(self):
        wf = self.workflow_create(self.wf_def)
        self.cron_trigger_create(
            "trigger", wf[0]["Name"], "{}", "5 * * * *")

        alt_trs = self.mistral_alt_user("cron-trigger-list")

        self.assertNotIn("trigger", [t["Name"] for t in alt_trs])


class ExecutionIsolationCLITests(base_v2.MistralClientTestBase):

    def test_execution_isolation(self):
        wf = self.workflow_create(self.wf_def)
        ex = self.execution_create(wf[0]["Name"])
        exec_id = self.get_field_value(ex, "ID")

        execs = self.mistral_admin("execution-list")
        self.assertIn(exec_id, [e["ID"] for e in execs])

        alt_execs = self.mistral_alt_user("execution-list")
        self.assertNotIn(exec_id, [e["ID"] for e in alt_execs])

    def test_get_execution_from_another_tenant(self):
        wf = self.workflow_create(self.wf_def)
        ex = self.execution_create(wf[0]["Name"])
        exec_id = self.get_field_value(ex, "ID")

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "execution-get",
            params=exec_id
        )


class EnvironmentIsolationCLITests(base_v2.MistralClientTestBase):

    def setUp(self):
        super(EnvironmentIsolationCLITests, self).setUp()

        self.env_file = "env.yaml"
        self.create_file("{0}".format(self.env_file),
                         "name: env\n"
                         "description: Test env\n"
                         "variables:\n"
                         "  var: value")

    def test_environment_name_uniqueness(self):
        self.environment_create(self.env_file)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_admin,
            "environment-create",
            params=self.env_file
        )

        self.environment_create(self.env_file, admin=False)

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "environment-create",
            params=self.env_file
        )

    def test_environment_isolation(self):
        env = self.environment_create(self.env_file)
        env_name = self.get_field_value(env, "Name")
        envs = self.mistral_admin("environment-list")

        self.assertIn(env_name, [en["Name"] for en in envs])

        alt_envs = self.mistral_alt_user("environment-list")

        self.assertNotIn(env_name, [en["Name"] for en in alt_envs])

    def test_get_env_from_another_tenant(self):
        env = self.environment_create(self.env_file)
        env_name = self.get_field_value(env, "Name")

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "environment-get",
            params=env_name
        )

    def test_delete_env_from_another_tenant(self):
        env = self.environment_create(self.env_file)
        env_name = self.get_field_value(env, "Name")

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "environment-delete",
            params=env_name
        )


class ActionExecutionIsolationCLITests(base_v2.MistralClientTestBase):

    def test_action_execution_isolation(self):
        wf = self.workflow_create(self.wf_def)
        wf_exec = self.execution_create(wf[0]["Name"])
        direct_ex_id = self.get_field_value(wf_exec, 'ID')

        self.wait_execution_success(direct_ex_id)

        act_execs = self.mistral_admin("action-execution-list")
        self.assertIn(wf[0]["Name"],
                      [act["Workflow name"] for act in act_execs])

        alt_act_execs = self.mistral_alt_user("action-execution-list")
        self.assertNotIn(wf[0]["Name"],
                         [act["Workflow name"] for act in alt_act_execs])

    def test_get_action_execution_from_another_tenant(self):
        wf = self.workflow_create(self.wf_def)
        ex = self.execution_create(wf[0]["Name"])
        exec_id = self.get_field_value(ex, "ID")

        self.assertRaises(
            exceptions.CommandFailed,
            self.mistral_alt_user,
            "action-execution-get",
            params=exec_id
        )



# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import copy
import urllib

from oslo_serialization import jsonutils

from keystoneauth1 import exceptions

from mistralclient import utils

urlparse = urllib.parse


class Resource(object):
    resource_name = 'Something'
    defaults = {}

    def __init__(self, manager, data):
        self.manager = manager
        self._data = data
        self._set_defaults()
        self._set_attributes()

    def _set_defaults(self):
        for k, v in self.defaults.items():
            if k not in self._data:
                self._data[k] = v

    def _set_attributes(self):
        for k, v in self._data.items():
            try:
                setattr(self, k, v)
            except AttributeError:
                # In this case we already defined the attribute on the class
                pass

    def to_dict(self):
        return copy.deepcopy(self._data)

    def __str__(self):
        values = ", ".join(
            ["%s='%s'" % (n, v) for n, v in self._data.items()]
        )

        return "%s [%s]" % (self.resource_name, values)


def _check_items(obj, searches):
    try:
        return all(getattr(obj, attr) == value for (attr, value) in searches)
    except AttributeError:
        return False


def extract_json(response, response_key):
    if response_key is not None:
        return get_json(response)[response_key]
    else:
        return get_json(response)


class ResourceManager(object):
    resource_class = None

    def __init__(self, http_client, enforce_raw_definitions=False):
        self.http_client = http_client
        self.enforce_raw_definitions = enforce_raw_definitions

    def get_contents_if_file(self, contents_or_file_name):
        if self.enforce_raw_definitions:
            return contents_or_file_name
        else:
            return utils.get_contents_if_file(contents_or_file_name)

    def find(self, **kwargs):
        return [i for i in self.list() if _check_items(i, kwargs.items())]

    def list(self):
        """This is an abstract method

        This is added here so that the find method gains some clarity.
        It must be implemented by the child class in order to find to work
        """
        raise NotImplementedError("abstract method list must be implemented")

    @staticmethod
    def _build_query_params(marker=None, limit=None, sort_keys=None,
                            sort_dirs=None, fields=None, filters=None,
                            scope=None, namespace=None):
        qparams = {}

        if marker:
            qparams['marker'] = marker

        if limit and limit > 0:
            qparams['limit'] = limit

        if sort_keys:
            qparams['sort_keys'] = sort_keys

        if sort_dirs:
            qparams['sort_dirs'] = sort_dirs

        if fields:
            qparams['fields'] = ",".join(fields)

        if filters:
            for name, val in filters.items():
                qparams[name] = val

        if scope:
            qparams['scope'] = scope

        if namespace:
            qparams['namespace'] = namespace

        return ("?%s" % urlparse.urlencode(list(qparams.items()))
                if qparams else "")

    def _ensure_not_empty(self, **kwargs):
        for name, value in kwargs.items():
            if value is None or (isinstance(value, str) and len(value) == 0):
                raise APIException(
                    400,
                    '%s is missing field "%s"' %
                    (self.resource_class.__name__, name)
                )

    def _validate(self, url, data, response_key=None, dump_json=True,
                  headers=None, is_iter_resp=False):
        return self._create(
            url,
            data,
            response_key,
            dump_json,
            headers,
            is_iter_resp,
            resp_status_ok=200,
            as_class=False
        )

    def _create(self, url, data, response_key=None, dump_json=True,
                headers=None, is_iter_resp=False, resp_status_ok=201,
                as_class=True):
        if dump_json:
            data = jsonutils.dumps(data)

        try:
            resp = self.http_client.post(url, data, headers)
        except exceptions.HttpError as ex:
            self._raise_api_exception(ex.response)

        if resp.status_code != resp_status_ok:
            self._raise_api_exception(resp)

        resource = extract_json(resp, response_key)

        if is_iter_resp:
            return [
                self.resource_class(self, resource_data)
                for resource_data in resource
            ]

        return self.resource_class(self, resource) if as_class else resource

    def _update(self, url, data, response_key=None, dump_json=True,
                headers=None, is_iter_resp=False):
        if dump_json:
            data = jsonutils.dumps(data)

        try:
            resp = self.http_client.put(url, data, headers)
        except exceptions.HttpError as ex:
            self._raise_api_exception(ex.response)

        if resp.status_code != 200:
            self._raise_api_exception(resp)

        resource = extract_json(resp, response_key)

        if is_iter_resp:
            return [
                self.resource_class(self, resource_data)
                for resource_data in resource
            ]

        return self.resource_class(self, resource)

    def _list(self, url, response_key=None, headers=None,
              returned_res_cls=None):

        try:
            resp = self.http_client.get(url, headers)
        except exceptions.HttpError as ex:
            self._raise_api_exception(ex.response)

        if resp.status_code != 200:
            self._raise_api_exception(resp)

        resource_class = returned_res_cls or self.resource_class

        return [
            resource_class(self, resource_data)
            for resource_data in extract_json(resp, response_key)
        ]

    def _get(self, url, response_key=None, headers=None):
        try:
            resp = self.http_client.get(url, headers)
        except exceptions.HttpError as ex:
            self._raise_api_exception(ex.response)

        if resp.status_code == 200:
            return self.resource_class(self, extract_json(resp, response_key))
        else:
            self._raise_api_exception(resp)

    def _delete(self, url, headers=None):
        try:
            resp = self.http_client.delete(url, headers)
        except exceptions.HttpError as ex:
            self._raise_api_exception(ex.response)

        if resp.status_code != 204:
            self._raise_api_exception(resp)

    @staticmethod
    def _raise_api_exception(resp):
        try:
            error_data = (
                resp.headers.get("Server-Error-Message", None)
                or get_json(resp).get("faultstring")
            )
        except ValueError:
            error_data = resp.content

        raise APIException(
            error_code=resp.status_code,
            error_message=error_data
        )


def get_json(response):
    """Gets JSON representation of response.

    This method provided backward compatibility with old versions
    of requests library.
    """
    json_field_or_function = getattr(response, 'json', None)

    if callable(json_field_or_function):
        return response.json()
    else:
        return jsonutils.loads(response.content)


class APIException(Exception):
    def __init__(self, error_code=None, error_message=None):
        super(APIException, self).__init__(error_message)
        self.error_code = error_code
        self.error_message = error_message



# Copyright 2016 - Catalyst IT Limited
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from osc_lib.command import command

from mistralclient.commands.v2 import base
from mistralclient import exceptions


class MemberFormatter(base.MistralFormatter):
    COLUMNS = [
        ('resource_id', 'Resource ID'),
        ('resource_type', 'Resource Type'),
        ('project_id', 'Resource Owner'),
        ('member_id', 'Member ID'),
        ('status', 'Status'),
        ('created_at', 'Created at'),
        ('updated_at', 'Updated at')
    ]

    @staticmethod
    def format(member=None, lister=False):
        if member:
            data = (
                member.resource_id,
                member.resource_type,
                member.project_id,
                member.member_id,
                member.status,
                member.created_at,
            )

            if hasattr(member, 'updated_at'):
                data += (member.updated_at,)
            else:
                data += (None,)
        else:
            data = (tuple('' for _ in range(len(MemberFormatter.COLUMNS))),)

        return MemberFormatter.headings(), data


class List(base.MistralLister):
    """List all members."""

    def _get_format_function(self):
        return MemberFormatter.format_list

    def get_parser(self, parsed_args):
        parser = super(List, self).get_parser(parsed_args)

        parser.add_argument(
            'resource_id',
            help='Resource id to be shared.'
        )
        parser.add_argument(
            'resource_type',
            help='Resource type.'
        )

        return parser

    def _get_resources(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine
        return mistral_client.members.list(
            parsed_args.resource_id,
            parsed_args.resource_type
        )


class Get(command.ShowOne):
    """Show specific member information."""

    def get_parser(self, prog_name):
        parser = super(Get, self).get_parser(prog_name)

        parser.add_argument(
            'resource',
            help='Resource ID to be shared.'
        )
        parser.add_argument(
            'resource_type',
            help='Resource type.'
        )
        parser.add_argument(
            '-m',
            '--member-id',
            default='',
            help='Project ID to whom the resource is shared to. No need to '
                 'provide this param if you are the resource member.'
        )

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine
        member = mistral_client.members.get(
            parsed_args.resource,
            parsed_args.resource_type,
            parsed_args.member_id,
        )

        return MemberFormatter.format(member)


class Create(command.ShowOne):
    """Shares a resource to another tenant."""

    def get_parser(self, prog_name):
        parser = super(Create, self).get_parser(prog_name)

        parser.add_argument(
            'resource_id',
            help='Resource ID to be shared.'
        )
        parser.add_argument(
            'resource_type',
            help='Resource type.'
        )
        parser.add_argument(
            'member_id',
            help='Project ID to whom the resource is shared to.'
        )

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine
        member = mistral_client.members.create(
            parsed_args.resource_id,
            parsed_args.resource_type,
            parsed_args.member_id,
        )

        return MemberFormatter.format(member)


class Delete(command.Command):
    """Delete a resource sharing relationship."""

    def get_parser(self, prog_name):
        parser = super(Delete, self).get_parser(prog_name)

        parser.add_argument(
            'resource',
            help='Resource ID to be shared.'
        )
        parser.add_argument(
            'resource_type',
            help='Resource type.'
        )
        parser.add_argument(
            'member_id',
            help='Project ID to whom the resource is shared to.'
        )

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        try:
            mistral_client.members.delete(
                parsed_args.resource,
                parsed_args.resource_type,
                parsed_args.member_id,
            )

            print(
                "Request to delete %s member %s has been accepted." %
                (parsed_args.resource_type, parsed_args.member_id)
            )
        except Exception as e:
            print(e)

            error_msg = "Unable to delete the specified member."
            raise exceptions.MistralClientException(error_msg)


class Update(command.ShowOne):
    """Update resource sharing status."""

    def get_parser(self, prog_name):
        parser = super(Update, self).get_parser(prog_name)

        parser.add_argument(
            'resource_id',
            help='Resource ID to be shared.'
        )
        parser.add_argument(
            'resource_type',
            help='Resource type.'
        )
        parser.add_argument(
            '-m',
            '--member-id',
            default='',
            help='Project ID to whom the resource is shared to. No need to '
                 'provide this param if you are the resource member.'
        )
        parser.add_argument(
            '-s',
            '--status',
            default='accepted',
            choices=['pending', 'accepted', 'rejected'],
            help='status of the sharing.'
        )

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        member = mistral_client.members.update(
            parsed_args.resource_id,
            parsed_args.resource_type,
            parsed_args.member_id,
            status=parsed_args.status
        )

        return MemberFormatter.format(member)



# Copyright 2013 - Mirantis, Inc.
# Copyright 2016 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import base64
import copy
import logging
import os

from oslo_utils import importutils

import requests


AUTH_TOKEN = 'auth_token'
SESSION = 'session'
CACERT = 'cacert'
CERT_FILE = 'cert'
CERT_KEY = 'key'
INSECURE = 'insecure'
PROJECT_ID = 'project_id'
USER_ID = 'user_id'
REGION_NAME = 'region_name'

TARGET_AUTH_TOKEN = 'target_auth_token'
TARGET_SESSION = 'target_session'
TARGET_AUTH_URI = 'target_auth_url'
TARGET_PROJECT_ID = 'target_project_id'
TARGET_USER_ID = 'target_user_id'
TARGET_INSECURE = 'target_insecure'
TARGET_SERVICE_CATALOG = 'target_service_catalog'
TARGET_REGION_NAME = 'target_region_name'
TARGET_USER_DOMAIN_NAME = 'target_user_domain_name'
TARGET_PROJECT_DOMAIN_NAME = 'target_project_domain_name'

osprofiler_web = importutils.try_import("osprofiler.web")

LOG = logging.getLogger(__name__)


def log_request(func):
    def decorator(self, *args, **kwargs):
        resp = func(self, *args, **kwargs)
        LOG.debug("HTTP %s %s %d", resp.request.method, resp.url,
                  resp.status_code)
        return resp
    return decorator


class HTTPClient(object):
    def __init__(self, base_url, **kwargs):
        self.base_url = base_url
        self.session = kwargs.get('session')
        if not self.session:
            self.session = requests.Session()
        self.auth_token = kwargs.get(AUTH_TOKEN)
        self.project_id = kwargs.get(PROJECT_ID)
        self.user_id = kwargs.get(USER_ID)
        self.cacert = kwargs.get(CACERT)
        self.insecure = kwargs.get(INSECURE, False)
        self.region_name = kwargs.get(REGION_NAME)
        self.ssl_options = {}

        self.target_session = kwargs.get(TARGET_SESSION)
        self.target_auth_token = kwargs.get(TARGET_AUTH_TOKEN)
        self.target_auth_uri = kwargs.get(TARGET_AUTH_URI)
        self.target_user_id = kwargs.get(TARGET_USER_ID)
        self.target_project_id = kwargs.get(TARGET_PROJECT_ID)
        self.target_service_catalog = kwargs.get(TARGET_SERVICE_CATALOG)
        self.target_region_name = kwargs.get(TARGET_REGION_NAME)
        self.target_insecure = kwargs.get(TARGET_INSECURE)
        self.target_user_domain_name = kwargs.get(TARGET_USER_DOMAIN_NAME)
        self.target_project_domain_name = kwargs.get(
            TARGET_PROJECT_DOMAIN_NAME
        )

        if self.base_url.startswith('https'):
            if self.cacert and not os.path.exists(self.cacert):
                raise ValueError('Unable to locate cacert file '
                                 'at %s.' % self.cacert)

            if self.cacert and self.insecure:
                LOG.warning('Client is set to not verify even though '
                            'cacert is provided.')

            if self.insecure:
                self.ssl_options['verify'] = False
            else:
                if self.cacert:
                    self.ssl_options['verify'] = self.cacert
                else:
                    self.ssl_options['verify'] = True

            self.ssl_options['cert'] = (
                kwargs.get(CERT_FILE),
                kwargs.get(CERT_KEY)
            )

    @log_request
    def get(self, url, headers=None):
        options = self._get_request_options('get', headers)

        return self.session.get(self.base_url + url, **options)

    @log_request
    def post(self, url, body, headers=None):
        options = self._get_request_options('post', headers)

        return self.session.post(self.base_url + url, data=body, **options)

    @log_request
    def put(self, url, body, headers=None):
        options = self._get_request_options('put', headers)

        return self.session.put(self.base_url + url, data=body, **options)

    @log_request
    def delete(self, url, headers=None):
        options = self._get_request_options('delete', headers)

        return self.session.delete(self.base_url + url, **options)

    def _get_request_options(self, method, headers):
        headers = self._update_headers(headers)

        if method in ['post', 'put']:
            content_type = headers.get('content-type', 'application/json')
            headers['content-type'] = content_type

        options = copy.deepcopy(self.ssl_options)
        options['headers'] = headers

        return options

    def _update_headers(self, headers):
        if not headers:
            headers = {}

        if isinstance(self.session, requests.Session):
            if self.auth_token:
                headers['X-Auth-Token'] = self.auth_token

            if self.project_id:
                headers['X-Project-Id'] = self.project_id

            if self.user_id:
                headers['X-User-Id'] = self.user_id

        if self.region_name:
            headers['X-Region-Name'] = self.region_name

        if self.target_auth_token:
            headers['X-Target-Auth-Token'] = self.target_auth_token

        if self.target_auth_uri:
            headers['X-Target-Auth-Uri'] = self.target_auth_uri

        if self.target_project_id:
            headers['X-Target-Project-Id'] = self.target_project_id

        if self.target_user_id:
            headers['X-Target-User-Id'] = self.target_user_id

        if self.target_insecure:
            # Note(akovi): due to changes in requests, this parameter
            # must be a string. Basically, it is a truthy value on
            # the server side.
            headers['X-Target-Insecure'] = str(self.target_insecure)

        if self.target_region_name:
            headers['X-Target-Region-Name'] = self.target_region_name

        if self.target_user_domain_name:
            headers['X-Target-User-Domain-Name'] = self.target_user_domain_name

        if self.target_project_domain_name:
            h_name = 'X-Target-Project-Domain-Name'

            headers[h_name] = self.target_project_domain_name

        if self.target_service_catalog:
            headers['X-Target-Service-Catalog'] = base64.b64encode(
                self.target_service_catalog.encode('utf-8')
            )

        if osprofiler_web:
            # Add headers for osprofiler.
            headers.update(osprofiler_web.get_trace_id_headers())

        return headers



# Copyright 2014 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo_serialization import jsonutils
from oslo_utils import uuidutils

from mistralclient.api import base


class CronTrigger(base.Resource):
    resource_name = 'CronTrigger'


class CronTriggerManager(base.ResourceManager):
    resource_class = CronTrigger

    def create(self, name, workflow_identifier, workflow_input=None,
               workflow_params=None, pattern=None,
               first_time=None, count=None):
        self._ensure_not_empty(
            name=name,
            workflow_identifier=workflow_identifier
        )

        data = {
            'name': name,
            'pattern': pattern,
            'first_execution_time': first_time,
            'remaining_executions': count
        }

        if uuidutils.is_uuid_like(workflow_identifier):
            data.update({'workflow_id': workflow_identifier})
        else:
            data.update({'workflow_name': workflow_identifier})

        if workflow_input:
            data.update({'workflow_input': jsonutils.dumps(workflow_input)})

        if workflow_params:
            data.update({'workflow_params': jsonutils.dumps(workflow_params)})

        return self._create('/cron_triggers', data)

    def list(self, marker='', limit=None, sort_keys='', fields='',
             sort_dirs='', **filters):
        query_string = self._build_query_params(
            marker=marker,
            limit=limit,
            sort_keys=sort_keys,
            sort_dirs=sort_dirs,
            fields=fields,
            filters=filters
        )

        return self._list('/cron_triggers%s' % query_string,
                          response_key='cron_triggers')

    def get(self, name):
        self._ensure_not_empty(name=name)

        return self._get('/cron_triggers/%s' % name)

    def delete(self, name):
        self._ensure_not_empty(name=name)

        self._delete('/cron_triggers/%s' % name)
